@@ -1,6 +1,6 @@
 --[[
 
-File l3build-check.lua Copyright (C) 2018,2019 The LaTeX3 Project
+File l3build-check.lua Copyright (C) 2018-2020 The LaTeX3 Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -215,15 +215,23 @@ local function normalize_log(content,engine,errlevels)
     if match(line, "^> \\box%d+=$") or match(line, "^> \\box%d+=(void)$") then
       line = gsub(line, "%d+=", "...=")
     end
-    -- Remove 'normal' direction information on boxes with (u)pTeX
-    line = gsub(line, ",? yoko direction,?", "")
-    line = gsub(line, ",? yoko%(math%) direction,?", "")
-    -- Remove '\displace 0.0' lines in (u)pTeX
-    if match(line,"^%.*\\displace 0%.0$") then
-      return ""
+    if not match(stdengine,"^e?u?ptex$") then
+      -- Remove 'normal' direction information on boxes with (u)pTeX
+      line = gsub(line, ",? yoko direction,?", "")
+      line = gsub(line, ",? yoko%(math%) direction,?", "")
+      -- Remove '\displace 0.0' lines in (u)pTeX
+      if match(line,"^%.*\\displace 0%.0$") then
+        return ""
+       end
      end
      -- Remove the \special line that in DVI mode keeps PDFs comparable
-    if match(line, "^%.*\\special%{pdf: docinfo << /Creator") then
+    if match(line, "^%.*\\special%{pdf: docinfo << /Creator") or
+      match(line, "^%.*\\special%{ps: /setdistillerparams") or
+      match(line, "^%.*\\special%{! <</........UUID") then
+      return ""
+    end
+     -- Remove \special lines for DVI .pro files
+    if match(line, "^%.*\\special%{header=") then
       return ""
     end
     if match(line, "^%.*\\special%{dvipdfmx:config") then
@@ -249,6 +257,9 @@ local function normalize_log(content,engine,errlevels)
     line = gsub(line, "save cache:", "load cache:")
     -- A tidy-up to keep LuaTeX and other engines in sync
     line = gsub(line, utf8_char(127), "^^?")
+    -- Remove lua data reference ids
+    line = gsub(line, "<lua data reference [0-9]+>",
+                      "<lua data reference ...>")
     -- Unicode engines display chars in the upper half of the 8-bit range:
     -- tidy up to match pdfTeX if an ASCII engine is in use
     if next(asciiengines) then
@@ -283,9 +294,14 @@ local function normalize_log(content,engine,errlevels)
   if recordstatus then
     new_content = new_content .. '***************' .. os_newline
     for i = 1, checkruns do
-      new_content = new_content ..
-        'Compilation ' .. i .. ' of test file completed with exit status ' ..
-        errlevels[i] .. os_newline
+      if (errlevels[i]==nil) then
+        new_content = new_content ..
+          'Compilation ' .. i .. ' of test file skipped ' .. os_newline
+      else
+        new_content = new_content ..
+          'Compilation ' .. i .. ' of test file completed with exit status ' ..
+          errlevels[i] .. os_newline
+      end
     end
   end
   return new_content
@@ -523,7 +539,10 @@ local function normalize_pdf(content)
       binary = false
       stream = true
       stream_content = "stream" .. os_newline
-    elseif not match(line, "^ *$") then
+    elseif not match(line, "^ *$") and
+      not match(line,"^%%%%Invocation") and 
+      not match(line,"^%%%%%+") then
+      line = gsub(line,"%/ID( ?)%[<[^>]+><[^>]+>]","/ID%1[<ID-STRING><ID-STRING>]")
       new_content = new_content .. line .. os_newline
     end
   end
@@ -749,11 +768,14 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
         .. setup(lvtfile)
         .. (hide and (" > " .. os_null) or "")
         .. os_concat ..
-      runtest_tasks(jobname(lvtfile))
+      runtest_tasks(jobname(lvtfile),i)
     )
     -- Break the loop if the result is stable
     if breakout and i < checkruns then
       if pdfmode then
+        if fileexists(testdir .. "/" .. name .. dviext) then
+          dvitopdf(name, testdir, engine, hide)
+        end
         rewrite(pdffile,npffile,normalize_pdf)
         if compare_pdf(name,engine,true) == 0 then
           break
@@ -796,7 +818,7 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
 end
 
 -- A hook to allow additional tasks to run for the tests
-runtest_tasks = runtest_tasks or function(name)
+runtest_tasks = runtest_tasks or function(name,run)
   return ""
 end
 
@@ -958,6 +980,10 @@ end
 function save(names)
   checkinit()
   local engines = options["engine"] or {stdengine}
+  if names == nil then
+    print("Arguments are required for the save command")
+    return 1
+  end
   for _,name in pairs(names) do
     if testexists(name) then
       for _,engine in pairs(engines) do
@@ -989,7 +1015,7 @@ function save(names)
         .. lveext .. " file of the same name")
       return 1
     else
-      print('Test "'.. name .. '"not found')
+      print('Test "' .. name .. '" not found')
       return 1
     end
   end
